@@ -1,6 +1,6 @@
 from django.conf.urls.defaults import *
 
-from admin import fiber_admin_site
+import fiber_admin
 import admin_views
 
 
@@ -8,5 +8,5 @@ urlpatterns = patterns('',
     url(r'^page/(?P<id>\d+)/move_up/$', admin_views.page_move_up, name='fiber_page_move_up'),
     url(r'^page/(?P<id>\d+)/move_down/$', admin_views.page_move_down, name='fiber_page_move_down'),
     url(r'^login/$', admin_views.fiber_login, name='fiber_login'),
-    (r'^fiber_admin/', include(fiber_admin_site.urls)),
+    (r'^fiber_admin/', include(fiber_admin.site.urls)),
 )
